@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1999-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1999-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -830,7 +830,7 @@ trace_receive(Process* receiver,
 }
 
 int
-seq_trace_update_send(Process *p)
+seq_trace_update_serial(Process *p)
 {
     ErtsTracer seq_tracer = erts_get_system_seq_tracer();
     ASSERT((is_tuple(SEQ_TRACE_TOKEN(p)) || is_nil(SEQ_TRACE_TOKEN(p))));
@@ -853,6 +853,18 @@ seq_trace_update_send(Process *p)
     return 1;
 }
 
+void
+erts_seq_trace_update_node_token(Eterm token)
+{
+    Eterm serial;
+    Uint serial_num;
+    SEQ_TRACE_T_SENDER(token) = erts_this_dist_entry->sysname;
+    serial = SEQ_TRACE_T_SERIAL(token);
+    serial_num = unsigned_val(serial);
+    serial_num++;
+    SEQ_TRACE_T_SERIAL(token) = make_small(serial_num);
+}
+
 
 /* Send a sequential trace message to the sequential tracer.
  * p is the caller (which contains the trace token), 
@@ -929,6 +941,9 @@ seq_trace_output_generic(Eterm token, Eterm msg, Uint type,
 #undef LOCAL_HEAP_SIZE
 }
 
+
+
+
 /* Send {trace_ts, Pid, return_to, {Mod, Func, Arity}, Timestamp}
  * or   {trace, Pid, return_to, {Mod, Func, Arity}}
  */
