@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2004-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -35,9 +35,10 @@
 	 map_field_lists/1,cover_bin_opt/1,
 	 val_dsetel/1,bad_tuples/1,bad_try_catch_nesting/1,
          receive_stacked/1,aliased_types/1,type_conflict/1,
-         infer_on_eq/1,infer_dead_value/1,
+         infer_on_eq/1,infer_dead_value/1,infer_on_ne/1,
+         branch_to_try_handler/1,call_without_stack/1,
          receive_marker/1,safe_instructions/1,
-         missing_return_type/1]).
+         missing_return_type/1,will_bif_succeed/1]).
 
 -include_lib("common_test/include/ct.hrl").
 
@@ -67,8 +68,10 @@ groups() ->
        map_field_lists,cover_bin_opt,val_dsetel,
        bad_tuples,bad_try_catch_nesting,
        receive_stacked,aliased_types,type_conflict,
-       infer_on_eq,infer_dead_value,receive_marker,
-       safe_instructions,missing_return_type]}].
+       infer_on_eq,infer_dead_value,infer_on_ne,
+       branch_to_try_handler,call_without_stack,
+       receive_marker,safe_instructions,
+       missing_return_type,will_bif_succeed]}].
 
 init_per_suite(Config) ->
     test_lib:recompile(?MODULE),
@@ -151,19 +154,34 @@ stack(Config) when is_list(Config) ->
 
 call_last(Config) when is_list(Config) ->
     Errors = do_val(call_last, Config),
-    [{{t,a,1},{{call_last,1,{f,8},2},9,{allocated,1}}},
+    [{{t,a,1},
+      {{call_last,1,{f,8},2},9,{allocated,1}}},
      {{t,b,1},
-      {{call_ext_last,2,{extfunc,lists,seq,2},2},
-       10,
-       {allocated,1}}}] = Errors,
+      {{call_ext_last,2,{extfunc,lists,seq,2},2},10,{allocated,1}}},
+     {{t,baz,2},
+      {{call_ext_only,2,{extfunc,erlang,put,2}},5,{allocated,0}}},
+     {{t,biz,2},
+      {{call_only,2,{f,10}},5,{allocated,0}}}] = Errors,
+    ok.
+
+call_without_stack(Config) when is_list(Config) ->
+    Errors = do_val(call_without_stack, Config),
+    [{{t,local,2},
+        {{call,2,{f,2}},4,{allocated,none}}},
+     {{t,remote,2},
+        {{call_ext,2,{extfunc,lists,seq,2}},4,{allocated,none}}}] = Errors,
     ok.
 
 merge_undefined(Config) when is_list(Config) ->
     Errors = do_val(merge_undefined, Config),
-    [{{t,handle_call,2},
+    [{{t,undecided,2},
       {{call_ext,2,{extfunc,debug,filter,2}},
        22,
-       {uninitialized_reg,{y,_}}}}] = Errors,
+       {allocated,undecided}}},
+     {{t,uninitialized,2},
+      {{call_ext,2,{extfunc,io,format,2}},
+       17,
+       {uninitialized_reg,{y,1}}}}] = Errors,
     ok.
 
 uninit(Config) when is_list(Config) ->
@@ -220,11 +238,11 @@ bad_catch_try(Config) when is_list(Config) ->
       {{catch_end,{x,9}},
        8,{invalid_tag_register,{x,9}}}},
      {{bad_catch_try,bad_3,1},
-      {{catch_end,{y,1}},9,{invalid_tag,{y,1},{atom,kalle}}}},
+      {{catch_end,{y,1}},9,{invalid_tag,{y,1},{t_atom,[kalle]}}}},
      {{bad_catch_try,bad_4,1},
       {{'try',{x,0},{f,15}},5,{invalid_tag_register,{x,0}}}},
      {{bad_catch_try,bad_5,1},
-      {{try_case,{y,1}},12,{invalid_tag,{y,1},term}}},
+      {{try_case,{y,1}},12,{invalid_tag,{y,1},any}}},
      {{bad_catch_try,bad_6,1},
       {{move,{integer,1},{y,1}},7,
        {invalid_store,{y,1}}}}] = Errors,
@@ -235,7 +253,7 @@ cons_guard(Config) when is_list(Config) ->
     [{{cons,foo,1},
       {{get_list,{x,0},{x,1},{x,2}},
        5,
-       {bad_type,{needed,cons},{actual,term}}}}] = Errors,
+       {bad_type,{needed,{t_cons,any,any}},{actual,any}}}}] = Errors,
     ok.
 
 freg_range(Config) when is_list(Config) ->
@@ -266,7 +284,7 @@ freg_uninit(Config) when is_list(Config) ->
        {uninitialized_reg,{fr,1}}}},
      {{t,sum_2,2},
       {{bif,fadd,{f,0},[{fr,0},{fr,1}],{fr,0}},
-       9,
+       10,
        {uninitialized_reg,{fr,0}}}}] = Errors,
     ok.
 
@@ -320,7 +338,7 @@ state_after_fault_in_catch(Config) when is_list(Config) ->
 no_exception_in_catch(Config) when is_list(Config) ->
     Errors = do_val(no_exception_in_catch, Config),
     [{{no_exception_in_catch,nested_of_1,4},
-      {{move,{x,3},{x,0}},87,{uninitialized_reg,{x,3}}}}] = Errors,
+      {{try_case_end,{x,0}},180,ambiguous_catch_try_state}}] = Errors,
     ok.
 
 undef_label(Config) when is_list(Config) ->
@@ -342,7 +360,7 @@ undef_label(Config) when is_list(Config) ->
 	 5},
     Errors = beam_val(M),
     [{{undef_label,t,1},{undef_labels,[42]}},
-     {{undef_label,x,1},{return,4,no_entry_label}}] = Errors,
+     {{undef_label,x,1},no_entry_label}] = Errors,
     ok.
 
 illegal_instruction(Config) when is_list(Config) ->
@@ -366,8 +384,8 @@ illegal_instruction(Config) when is_list(Config) ->
     Errors = beam_val(M),
     [{{illegal_instruction,t,1},
       {{my_illegal_instruction,{x,0}},4,unknown_instruction}},
-     {{'_',x,1},{bad_func_info,1,illegal_instruction}},
-     {{'_',y,0},{[],0,illegal_instruction}}] = Errors,
+     {{illegal_instruction,x,1},invalid_function_header},
+     {{illegal_instruction,y,0},invalid_function_header}] = Errors,
     ok.
 
 %% The beam_validator used to assume that a GC guard BIF could
@@ -404,13 +422,12 @@ process_request_bar(Pid, [Response]) when is_pid(Pid) ->
 map_field_lists(Config) ->
     Errors = do_val(map_field_lists, Config),
     [{{map_field_lists,x,1},
-      {{test,has_map_fields,{f,1},{x,0},
-	{list,[{atom,a},{atom,a}]}},
-       5,
+      {{test,has_map_fields,{f,1},{x,0},{list,[{atom,a},{atom,a}]}},
+       6,
        keys_not_unique}},
      {{map_field_lists,y,1},
       {{test,has_map_fields,{f,3},{x,0},{list,[]}},
-       5,
+       6,
        empty_field_list}}
     ] = Errors.
 
@@ -519,13 +536,13 @@ destroy_reg({Tag,N}) ->
 bad_tuples(Config) ->
     Errors = do_val(bad_tuples, Config),
     [{{bad_tuples,heap_overflow,1},
-      {{put,{x,0}},8,{heap_overflow,{left,0},{wanted,1}}}},
+      {{put,{x,0}},9,{heap_overflow,{left,0},{wanted,1}}}},
      {{bad_tuples,long,2},
-      {{put,{atom,too_long}},8,not_building_a_tuple}},
+      {{put,{atom,too_long}},9,not_building_a_tuple}},
      {{bad_tuples,self_referential,1},
-      {{put,{x,1}},7,{tuple_in_progress,{x,1}}}},
+      {{put,{x,1}},8,{unfinished_tuple,{x,1}}}},
      {{bad_tuples,short,1},
-      {{move,{x,1},{x,0}},7,{tuple_in_progress,{x,1}}}}] = Errors,
+      {{move,{x,1},{x,0}},8,{unfinished_tuple,{x,1}}}}] = Errors,
 
     ok.
 
@@ -533,7 +550,7 @@ bad_try_catch_nesting(Config) ->
     Errors = do_val(bad_try_catch_nesting, Config),
     [{{bad_try_catch_nesting,main,2},
       {{'try',{y,2},{f,3}},
-       7,
+       8,
        {bad_try_catch_nesting,{y,2},[{{y,1},{trytag,[5]}}]}}}] = Errors,
     ok.
 
@@ -542,37 +559,37 @@ receive_stacked(Config) ->
     Errors = do_val(Mod, Config),
     [{{receive_stacked,f1,0},
       {{loop_rec_end,{f,3}},
-       17,
+       18,
        {fragile_message_reference,{y,_}}}},
      {{receive_stacked,f2,0},
-      {{test_heap,3,0},10,{fragile_message_reference,{y,_}}}},
+      {{test_heap,3,0},11,{fragile_message_reference,{y,_}}}},
      {{receive_stacked,f3,0},
-      {{test_heap,3,0},10,{fragile_message_reference,{y,_}}}},
+      {{test_heap,3,0},11,{fragile_message_reference,{y,_}}}},
      {{receive_stacked,f4,0},
-      {{test_heap,3,0},10,{fragile_message_reference,{y,_}}}},
+      {{test_heap,3,0},11,{fragile_message_reference,{y,_}}}},
      {{receive_stacked,f5,0},
       {{loop_rec_end,{f,23}},
-       23,
+       24,
        {fragile_message_reference,{y,_}}}},
      {{receive_stacked,f6,0},
       {{gc_bif,byte_size,{f,29},0,[{y,_}],{x,0}},
-       12,
+       13,
        {fragile_message_reference,{y,_}}}},
      {{receive_stacked,f7,0},
       {{loop_rec_end,{f,33}},
-       20,
+       21,
        {fragile_message_reference,{y,_}}}},
      {{receive_stacked,f8,0},
       {{loop_rec_end,{f,38}},
-       20,
+       21,
        {fragile_message_reference,{y,_}}}},
      {{receive_stacked,m1,0},
       {{loop_rec_end,{f,43}},
-       19,
+       20,
        {fragile_message_reference,{y,_}}}},
      {{receive_stacked,m2,0},
       {{loop_rec_end,{f,48}},
-       33,
+       34,
        {fragile_message_reference,{y,_}}}}] = Errors,
 
     %% Compile the original source code as a smoke test.
@@ -708,6 +725,25 @@ idv_1({_A, _B, _C, _D, _E, F, G},
 idv_1(_A, _B) ->
     error.
 
+%% ERL-998; type inference for select_val (#b_switch{}) was more clever than
+%% that for is_ne_exact (#b_br{}), sometimes failing validation when the type
+%% optimization pass acted on the former and the validator got the latter.
+
+-record(ion, {state}).
+
+infer_on_ne(Config) when is_list(Config) ->
+    #ion{state = closing} = ion_1(#ion{ state = id(open) }),
+    #ion{state = closing} = ion_close(#ion{ state = open }),
+    ok.
+
+ion_1(State = #ion{state = open}) -> ion_2(State);
+ion_1(State = #ion{state = closing}) -> ion_2(State).
+
+ion_2(State = #ion{state = open}) -> ion_close(State);
+ion_2(#ion{state = closing}) -> ok.
+
+ion_close(State = #ion{}) -> State#ion{state = closing}.
+
 %% ERL-995: The first solution to ERIERL-348 was incomplete and caused
 %% validation to fail when living values depended on delayed type inference on
 %% "dead" values.
@@ -725,6 +761,17 @@ idv_2(State) ->
 
 idv_called_once(_State) -> ok.
 
+%% Direct jumps to try/catch handlers crash the emulator and must fail
+%% validation. This is provoked by OTP-15945.
+
+branch_to_try_handler(Config) ->
+    Errors = do_val(branch_to_try_handler, Config),
+    [{{branch_to_try_handler,main,1},
+      {{bif,tuple_size,{f,3},[{y,0}],{x,0}},
+       13,
+       {illegal_branch,try_handler,3}}}] = Errors,
+    ok.
+
 receive_marker(Config) when is_list(Config) ->
     Errors = do_val(receive_marker, Config),
 
@@ -794,7 +841,7 @@ do_val(Mod, Config) ->
 
 beam_val(M) ->
     Name = atom_to_list(element(1, M)),
-    {error,[{Name,Errors0}]} = beam_validator:module(M, []),
+    {error,[{Name,Errors0}]} = beam_validator:validate(M, strong),
     Errors = [E || {beam_validator,E} <- Errors0],
     _ = [io:put_chars(beam_validator:format_error(E)) ||
 	    E <- Errors],
@@ -820,5 +867,22 @@ night(Turned) ->
 
 participating(_, _, _, _) -> ok.
 
+%% map_get was known as returning 'none', but 'will_succeed' still returned
+%% 'maybe' causing validation to continue, eventually exploding when the 'none'
+%% value was used.
+will_bif_succeed(_Config) ->
+    ok = f1(body).
+
+%% +no_ssa_opt
+f1(body) when map_get(girl, #{friend => node()}); [], community ->
+    case $q and $K of
+        _V0 ->
+            0.1825965401179273;
+        0 ->
+            state#{[] => 0.10577334580729858, $J => 0}
+    end;
+f1(body) ->
+    ok.
+
 id(I) ->
     I.
