@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2004-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2020. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -24,13 +24,28 @@
 
 -include_lib("common_test/include/ct.hrl").
 
-all() -> [encrypt_decrypt__crypto_one_time,
-          prop__crypto_init_update
+all() -> [encrypt_decrypt_one_time,
+          init_update,
+          init_update_multi
          ].
 
 %%% First prepare Config and compile the property tests for the found tool:
 init_per_suite(Config) ->
-    ct_property_test:init_per_suite(Config).
+    case
+        try crypto:start() of
+            ok -> true;
+            {error, already_started} -> true;
+            _ -> false
+        catch
+            _:_ -> false
+        end
+    of
+        true ->
+            ct_property_test:init_per_suite(Config);
+        false ->
+	    {skip, "Crypto did not start"}
+    end.
+
 
 end_per_suite(Config) ->
     Config.
@@ -38,13 +53,21 @@ end_per_suite(Config) ->
 %%%================================================================
 %%% Test suites
 %%%
-encrypt_decrypt__crypto_one_time(Config) ->
+encrypt_decrypt_one_time(Config) ->
     ct_property_test:quickcheck(
       crypto_ng_api:prop__crypto_one_time(),
       Config
      ).
-prop__crypto_init_update(Config) ->
+
+init_update(Config) ->
     ct_property_test:quickcheck(
-      crypto_ng_api:prop__crypto_init_update(),
+      crypto_ng_api:prop__crypto_init_update_final(),
       Config
      ).
+
+init_update_multi(Config) ->
+    ct_property_test:quickcheck(
+      crypto_ng_api_stateful:prop__crypto_init_multi(Config),
+      Config
+     ).
+
