@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2004-2018. All Rights Reserved.
+ * Copyright Ericsson AB 2004-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,10 +19,7 @@
  */
 
 #include <string.h>
-
-#ifdef VXWORKS
-#include "reclaim.h"
-#endif
+#include <stdlib.h>
 
 #include "ei_runner.h"
 
@@ -31,15 +28,9 @@
  * Author:  Kent
  */
 
-#ifdef VXWORKS
-#define MESSAGE_BACK(SIZE) \
-    message("err = %d, size2 = %d, expected size = %d", \
-             err, size1, SIZE); 
-#else
 #define MESSAGE_BACK(SIZE) \
     message("err = %d, size2 = %d, expected size = %d, long long val = %lld", \
              err, size1, SIZE, (EI_LONGLONG)p); 
-#endif
 
 #define ERLANG_ANY (ERLANG_ASCII|ERLANG_LATIN1|ERLANG_UTF8)
 
@@ -483,7 +474,6 @@ TESTCASE(test_ei_decode_longlong)
 {
     ei_init();
 
-#ifndef VXWORKS
     EI_DECODE_2     (decode_longlong,  2, EI_LONGLONG, 0);
     EI_DECODE_2     (decode_longlong,  2, EI_LONGLONG, 255);
     EI_DECODE_2     (decode_longlong,  5, EI_LONGLONG, 256);
@@ -509,7 +499,6 @@ TESTCASE(test_ei_decode_longlong)
     EI_DECODE_2_FAIL(decode_longlong, 11, EI_LONGLONG,  ll(0xffffffffffffffff));
 
     EI_DECODE_2_FAIL(decode_longlong,  1, EI_LONGLONG,  0); /* Illegal type */
-#endif
     report(1);
 }
 
@@ -519,7 +508,6 @@ TESTCASE(test_ei_decode_ulonglong)
 {
     ei_init();
 
-#ifndef VXWORKS
     EI_DECODE_2     (decode_ulonglong, 2, EI_ULONGLONG, 0);
     EI_DECODE_2     (decode_ulonglong, 2, EI_ULONGLONG, 255);
     EI_DECODE_2     (decode_ulonglong, 5, EI_ULONGLONG, 256);
@@ -545,7 +533,6 @@ TESTCASE(test_ei_decode_ulonglong)
     EI_DECODE_2     (decode_ulonglong,11, EI_ULONGLONG,  ll(0xffffffffffffffff));
 
     EI_DECODE_2_FAIL(decode_ulonglong, 1, EI_ULONGLONG, 0); /* Illegal type */
-#endif
     report(1);
 }
 
@@ -637,8 +624,6 @@ TESTCASE(test_ei_decode_nonoptimal)
 
     /* ---------------------------------------------------------------- */
 
-#ifndef VXWORKS
-
     EI_DECODE_2(decode_longlong,  2, EI_LONGLONG, 42);
     EI_DECODE_2(decode_longlong,  5, EI_LONGLONG, 42);
     EI_DECODE_2(decode_longlong,  4, EI_LONGLONG, 42);
@@ -681,8 +666,6 @@ TESTCASE(test_ei_decode_nonoptimal)
 /*  EI_DECODE_2(decode_ulonglong, EI_ULONGLONG, -42); */
 /*  EI_DECODE_2(decode_ulonglong, EI_ULONGLONG, -42); */
 
-#endif /* !VXWORKS */
-
     /* ---------------------------------------------------------------- */
 
     report(1);
@@ -771,6 +754,192 @@ TESTCASE(test_ei_decode_utf8_atom)
 
 /* ******************************************************************** */
 
+TESTCASE(test_ei_decode_iodata)
+{
+    char *buf = NULL, *data = NULL;
+    ei_init();
+
+    while (1) {
+        int unexpected_write = 0;
+        int i;
+        int len, index, saved_index, err;
+
+        if (buf)
+            free_packet(buf);
+        buf = read_packet(&len);
+
+        if (len == 4
+            && buf[0] == 'd'
+            && buf[1] == 'o'
+            && buf[2] == 'n'
+            && buf[3] == 'e') {
+            break;
+        }
+        
+        index = 0;
+        err = ei_decode_version(buf, &index, NULL);
+        if (err != 0) {
+            free_packet(buf);
+            fail1("ei_decode_version returned %d", err);
+        }
+        saved_index = index;
+        err = ei_decode_iodata(buf, &index, &len, NULL);
+        if (err != 0) {
+            ei_x_buff x;
+            ei_x_new_with_version(&x);
+            ei_x_encode_atom(&x, "decode_size_failed");
+            send_bin_term(&x);
+            ei_x_free(&x);
+            continue;
+        }
+        if (data) {
+            data -= 100;
+            free(data);
+        }
+        data = malloc(len + 200);
+        if (!data) {
+            ei_x_buff x;
+            ei_x_new_with_version(&x);
+            ei_x_encode_atom(&x, "malloc_failed");
+            send_bin_term(&x);
+            ei_x_free(&x);
+            continue;
+        }
+        for (i = 0; i < len + 200; i++)
+            data[i] = 'Y';
+        data += 100;
+        err = ei_decode_iodata(buf, &saved_index, NULL, (unsigned char *) data);
+        if (err != 0) {
+            ei_x_buff x;
+            ei_x_new_with_version(&x);
+            ei_x_encode_atom(&x, "decode_data_failed");
+            send_bin_term(&x);
+            ei_x_free(&x);
+            continue;
+        }
+
+        for (i = -100; i < 0; i++) {
+            if (data[i] != 'Y') {
+                ei_x_buff x;
+                ei_x_new_with_version(&x);
+                ei_x_encode_atom(&x, "unexpected_write_before_data");
+                send_bin_term(&x);
+                ei_x_free(&x);
+                unexpected_write = !0;
+                break;
+            }
+        }
+
+        if (!unexpected_write) {
+            for (i = len; i < len + 100; i++) {
+                if (data[i] != 'Y') {
+                    ei_x_buff x;
+                    ei_x_new_with_version(&x);
+                    ei_x_encode_atom(&x, "unexpected_write_after_data");
+                    send_bin_term(&x);
+                    ei_x_free(&x);
+                    unexpected_write = !0;
+                    break;
+                }
+            }
+        }
+
+        if (!unexpected_write)
+            send_buffer(data, len);
+    }
+
+    if (buf)
+        free_packet(buf);
+    if (data) {
+        data -= 100;
+        free(data);
+    }
+    report(1);
+}
+
+/* ******************************************************************** */
+
+/*
+ * Does not belong here move to its own suite...
+ */
+TESTCASE(test_ei_cmp_nc)
+{
+    char *buf = NULL;
+    ei_init();
+
+    while (1) {
+        int len, index, arity;
+        char atom[MAXATOMLEN_UTF8];
+        ei_x_buff x;
+
+        if (buf)
+            free_packet(buf);
+        buf = read_packet(&len);
+
+        if (len == 4
+            && buf[0] == 'd'
+            && buf[1] == 'o'
+            && buf[2] == 'n'
+            && buf[3] == 'e') {
+            break;
+        }
+
+        ei_x_new_with_version(&x);
+        index = 0;
+        if (ei_decode_version(buf, &index, NULL)
+            || ei_decode_tuple_header(buf, &index, &arity)
+            || (arity != 3)
+            || ei_decode_atom(buf, &index, atom)) {
+            ei_x_encode_atom(&x, "decode_tuple_failed");
+        }
+        else if (strcmp(atom, "cmp_pids") == 0) {
+            erlang_pid a, b;
+            if (ei_decode_pid(buf, &index, &a)
+                || ei_decode_pid(buf, &index, &b)) {
+                ei_x_encode_atom(&x, "decode_pids_failed");
+            }
+            else {
+                long res = (long) ei_cmp_pids(&a, &b);
+                ei_x_encode_long(&x, res);
+            }
+        }
+        else if (strcmp(atom, "cmp_ports") == 0) {
+            erlang_port a, b;
+            if (ei_decode_port(buf, &index, &a)
+                || ei_decode_port(buf, &index, &b)) {
+                ei_x_encode_atom(&x, "decode_ports_failed");
+            }
+            else {
+                long res = (long) ei_cmp_ports(&a, &b);
+                ei_x_encode_long(&x, res);
+            }
+        }
+        else if (strcmp(atom, "cmp_refs") == 0) {
+            erlang_ref a, b;
+            if (ei_decode_ref(buf, &index, &a)
+                || ei_decode_ref(buf, &index, &b)) {
+                ei_x_encode_atom(&x, "decode_refs_failed");
+            }
+            else {
+                long res = (long) ei_cmp_refs(&a, &b);
+                ei_x_encode_long(&x, res);
+            }
+        }
+        else {
+            ei_x_encode_atom(&x, "unexpected_operation");
+        }
+
+        send_bin_term(&x);
+        ei_x_free(&x);
+    }
+
+    if (buf)
+        free_packet(buf);
+    report(1);
+}
+
+/* ******************************************************************** */
+
 int ei_decode_my_atom_as(const char *buf, int *index, char *to,
 			 struct my_atom *atom) {
   erlang_char_encoding was,result;
