@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -41,7 +41,7 @@
 %%--------------------------------------------------------------------
 
 suite() ->
-    [{timetrap,{seconds,60}}].
+    [{timetrap,{seconds,90}}].
 
 all() ->
 %%    [check_docker_present] ++
@@ -196,17 +196,19 @@ login_otp_is_client(Config) ->
                                             [' ']
                                     end
         ],
-                                        
+
     chk_all_algos(?FUNCTION_NAME, CommonAuths, Config,
                   fun(AuthMethod,Alg) ->
                           {Opts,Dir} =
                               case AuthMethod of
                                   publickey ->
-                                      {[], setup_remote_auth_keys_and_local_priv(Alg, Config)};
+                                      {[{pref_public_key_algs, [Alg]}],
+                                       setup_remote_auth_keys_and_local_priv(Alg, Config)};
                                   _ ->
                                       {[{password,?PASSWD}], new_dir(Config)}
                               end,
                           ssh:connect(IP, Port, [{auth_methods, atom_to_list(AuthMethod)},
+                                                 {preferred_algorithms, ssh_transport:supported_algorithms()},
                                                  {user,?USER},
                                                  {user_dir, Dir},
                                                  {silently_accept_hosts,true},
@@ -234,7 +236,9 @@ login_otp_is_server(Config) ->
                           {Opts,UsrDir} =
                               case AuthMethod of
                                   publickey ->
-                                      {[{user_passwords, [{?USER,?BAD_PASSWD}]}],
+                                      {[{user_passwords, [{?USER,?BAD_PASSWD}]},
+                                        {pref_public_key_algs, [Alg]}
+                                       ],
                                        setup_remote_priv_and_local_auth_keys(Alg, Config)
                                       };
                                   _ ->
@@ -245,6 +249,7 @@ login_otp_is_server(Config) ->
                           {Server, Host, HostPort} =
                               ssh_test_lib:daemon(0,
                                                   [{auth_methods, atom_to_list(AuthMethod)},
+                                                   {preferred_algorithms, ssh_transport:supported_algorithms()},
                                                    {system_dir, SysDir},
                                                    {user_dir, UsrDir},
                                                    {failfun, fun ssh_test_lib:failfun/2}
@@ -264,13 +269,16 @@ all_algorithms_sftp_exec_reneg_otp_is_client(Config) ->
     {IP,Port} = ip_port(Config),
     chk_all_algos(?FUNCTION_NAME, CommonAlgs, Config,
                   fun(Tag, Alg) ->
+                          PrefAlgs =
+                              [{T,L} || {T,L} <- ssh_transport:supported_algorithms(),
+                                        T =/= Tag],
                           ConnRes =
                               ssh:connect(IP, Port, 
                                           [{user,?USER},
                                            {password,?PASSWD},
                                            {auth_methods, "password"},
                                            {user_dir, new_dir(Config)},
-                                           {preferred_algorithms, [{Tag,[Alg]}]},
+                                           {preferred_algorithms, [{Tag,[Alg]} | PrefAlgs]},
                                            {silently_accept_hosts,true},
                                            {user_interaction,false}
                                           ])  ,
@@ -293,11 +301,14 @@ all_algorithms_sftp_exec_reneg_otp_is_server(Config) ->
                                            public_key -> Alg;
                                            _ -> 'ssh-rsa'
                                        end,
+                          PrefAlgs =
+                              [{T,L} || {T,L} <- ssh_transport:supported_algorithms(),
+                                        T =/= Tag],
                           SftpRootDir = new_dir(Config),
                           %% ct:log("Rootdir = ~p",[SftpRootDir]),
                           {Server, Host, HostPort} =
                               ssh_test_lib:daemon(0,
-                                                  [{preferred_algorithms, [{Tag,[Alg]}]},
+                                                  [{preferred_algorithms, [{Tag,[Alg]} | PrefAlgs]},
                                                    {system_dir, setup_local_hostdir(HostKeyAlg, Config)},
                                                    {user_dir, UserDir},
                                                    {user_passwords, [{?USER,?PASSWD}]},
@@ -330,6 +341,7 @@ send_recv_big_with_renegotiate_otp_is_client(Config) ->
                                     {password,?PASSWD},
                                     {user_dir, setup_remote_auth_keys_and_local_priv('ssh-rsa', Config)},
                                     {silently_accept_hosts,true},
+                                    {preferred_algorithms, ssh_transport:supported_algorithms()},
                                     {user_interaction,false}
                                    ]),
 
@@ -438,6 +450,7 @@ exec_from_docker(Config, HostIP, HostPort, Command, Expects, ExtraSshArg) when i
                          [{user,?USER},
                           {password,?PASSWD},
                           {user_dir, new_dir(Config)},
+                          {preferred_algorithms, ssh_transport:supported_algorithms()},
                           {silently_accept_hosts,true},
                           {user_interaction,false}
                          ]),
@@ -521,21 +534,22 @@ result_of_exec(C, Ch, ExitStatus, Acc) ->
 chk_all_algos(FunctionName, CommonAlgs, Config, DoTestFun) when is_function(DoTestFun,2) ->
     ct:comment("~p algorithms",[length(CommonAlgs)]),
     %% Check each algorithm
-    Failed =
+    Nmax = length(CommonAlgs),
+    {_N,Failed} =
         lists:foldl(
-          fun({Tag,Alg}, FailedAlgos) ->
-                  %% ct:log("Try ~p",[Alg]),
+          fun({Tag,Alg}, {N,FailedAlgos}) ->
+                  ct:log("Try ~p  ~p/~p",[{Tag,Alg},N,Nmax]),
                   case DoTestFun(Tag,Alg) of
                       {ok,C} ->
                           ssh:close(C),
-                          FailedAlgos;
+                          {N+1,FailedAlgos};
                       ok ->
-                          FailedAlgos;
+                          {N+1,FailedAlgos};
                       Other ->
                           ct:log("FAILED! ~p ~p: ~p",[Tag,Alg,Other]),
-                          [{Alg,Other}|FailedAlgos]
+                          {N+1, [{Alg,Other}|FailedAlgos]}
                   end
-          end, [], CommonAlgs),
+          end, {1,[]}, CommonAlgs),
     ct:pal("~s", [format_result_table_use_all_algos(FunctionName, Config, CommonAlgs, Failed)]),
     case Failed of
         [] ->
@@ -612,6 +626,7 @@ setup_remote_auth_keys_and_local_priv(KeyAlg, IP, Port, UserDir, Config) ->
                                                    {password, ?PASSWD   },
                                                    {auth_methods, "password"},
                                                    {silently_accept_hosts,true},
+                                                   {preferred_algorithms, ssh_transport:supported_algorithms()},
                                                    {user_interaction,false}
                                                   ]),
     _ = ssh_sftp:make_dir(Ch, ".ssh"),
@@ -643,6 +658,7 @@ setup_remote_priv_and_local_auth_keys(KeyAlg, IP, Port, UserDir, Config) ->
     {ok,Ch,Cc} = ssh_sftp:start_channel(IP, Port, [{user,     ?USER  },
                                                    {password, ?PASSWD   },
                                                    {auth_methods, "password"},
+                                                   {preferred_algorithms, ssh_transport:supported_algorithms()},
                                                    {silently_accept_hosts,true},
                                                    {user_interaction,false}
                                                   ]),
@@ -920,14 +936,29 @@ find_common_algs(Remote, Local) ->
 use_algorithms(RemoteHelloBin) ->
     MyAlgos = ssh:chk_algos_opts(
                 [{modify_algorithms,
-                  [{append,
-                    [{kex,['diffie-hellman-group1-sha1']}
-                    ]}
+                  [{append, alg_diff()}
                   ]}
                 ]),
     ssh_transport:adjust_algs_for_peer_version(binary_to_list(RemoteHelloBin)++"\r\n",
                                                MyAlgos).
 
+
+alg_class_diff(Tag) ->
+    alg_diff(proplists:get_value(Tag, ssh:default_algorithms()),
+             proplists:get_value(Tag, ssh_transport:supported_algorithms())).
+
+alg_diff() ->
+    alg_diff(ssh:default_algorithms(), ssh_transport:supported_algorithms()).
+
+alg_diff(L1, L2) when is_atom(hd(L1)) ; is_atom(hd(L2))  ->
+    (L2--L1)--['AEAD_AES_256_GCM','AEAD_AES_128_GCM'];
+alg_diff(L1, L2) ->
+    [{T, Diff} || {{T,EL1},{T,EL2}} <- lists:zip(L1,L2),
+                  Diff <- [alg_diff(EL1,EL2)],
+                  Diff =/= []
+    ].
+
+
 kexint_msg2default_algorithms(#ssh_msg_kexinit{kex_algorithms = Kex,
                                                server_host_key_algorithms = PubKey,
                                                encryption_algorithms_client_to_server = CipherC2S,
@@ -1181,6 +1212,7 @@ call_sftp_in_docker(Config, ServerIP, ServerPort, Cmnds, UserDir) ->
                          [{user,?USER},
                           {password,?PASSWD},
                           {user_dir, UserDir},
+                          {preferred_algorithms, ssh_transport:supported_algorithms()},
                           {silently_accept_hosts,true},
                           {user_interaction,false}
                          ]),
