@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2019-2019. All Rights Reserved.
+%% Copyright Ericsson AB 2019-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -35,7 +35,6 @@
 %%--------------------------------------------------------------------
 
 all() ->
-    %% Note: ALPN not supported in sslv3
     case ssl_test_lib:openssl_sane_dtls_alpn() of 
         true ->
             [
@@ -46,7 +45,9 @@ all() ->
              {group, 'dtlsv1.2'},
              {group, 'dtlsv1'}];
         false ->
-            [{group, 'tlsv1.2'},
+            [
+             {group, 'tlsv1.3'},
+             {group, 'tlsv1.2'},
              {group, 'tlsv1.1'},
              {group, 'tlsv1'}]
     end.
@@ -117,30 +118,10 @@ end_per_suite(_Config) ->
     ssl_test_lib:kill_openssl().
 
 init_per_group(GroupName, Config) ->
-    case ssl_test_lib:is_tls_version(GroupName) of
-        true ->
-            case ssl_test_lib:supports_ssl_tls_version(GroupName) of
-                 true ->
-                    case ssl_test_lib:check_sane_openssl_version(GroupName) of
-                         true ->
-                            ssl_test_lib:init_tls_version(GroupName, Config);
-                         false ->
-                            {skip, openssl_does_not_support_version}
-                    end;
-                false ->
-                    {skip, openssl_does_not_support_version}
-            end; 
-         _ ->
-            Config
-    end.
+    ssl_test_lib:init_per_group_openssl(GroupName, Config).
 
 end_per_group(GroupName, Config) ->
-    case ssl_test_lib:is_tls_version(GroupName) of
-        true ->
-            ssl_test_lib:clean_tls_version(Config);
-       false ->
-            Config
-    end.
+    ssl_test_lib:end_per_group(GroupName, Config).
 
 init_per_testcase(TestCase, Config) -> 
     ct:timetrap({seconds, 30}),
@@ -148,14 +129,19 @@ init_per_testcase(TestCase, Config) ->
 
 special_init(erlang_client_alpn_openssl_server_alpn_renegotiate, Config) ->
     {ok, Version} = application:get_env(ssl, protocol_version),
-    ssl_test_lib:check_sane_openssl_renegotaite(Config, Version);
+    case ssl_test_lib:check_sane_openssl_renegotiate(Config, Version) of
+        {skip, _} = Skip ->
+            Skip;
+        Config ->
+            ssl_test_lib:openssl_allows_server_renegotiate(Config)
+    end;
 special_init(erlang_server_alpn_openssl_client_alpn_renegotiate, Config) ->
     {ok, Version} = application:get_env(ssl, protocol_version),
-    case ssl_test_lib:check_sane_openssl_renegotaite(Config, Version) of
+    case ssl_test_lib:check_sane_openssl_renegotiate(Config, Version) of
+        {skip, _} = Skip ->
+            Skip;
         Config ->
-            ssl_test_lib:openssl_allows_client_renegotaite(Config);
-        Skip ->
-            Skip
+            ssl_test_lib:openssl_allows_client_renegotiate(Config)
     end;
 special_init(_, Config) ->
      Config.
@@ -313,7 +299,7 @@ start_erlang_client_and_openssl_server_for_alpn_negotiation(Config, Data, Callba
                     {from, self()},
                     {mfa, {ssl_test_lib,
                            erlang_ssl_receive_and_assert_negotiated_protocol, [<<"spdy/2">>, Data]}},
-                    {options,  [{reuse_sessions, false} | ClientOpts]}]),
+                    {options, ClientOpts}]),
 
     Callback(Client, OpensslPort),
 
@@ -382,7 +368,7 @@ start_erlang_client_and_openssl_server_for_alpn_npn_negotiation(Config, Data, Ca
                     {from, self()},
                     {mfa, {ssl_test_lib,
                            erlang_ssl_receive_and_assert_negotiated_protocol, [<<"spdy/2">>, Data]}},
-                    {options, [{reuse_sessions, false} | ClientOpts]}]),
+                    {options, ClientOpts}]),
 
     Callback(Client, OpensslPort),
 
